#define PIPE_ADDRESS_0 0xb4b5b600

#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdio.h>
#include <util/delay.h>

#include "uart.h"
#include "wireless.h"

// UART file descriptor
// putchar and getchar are in uart.c
FILE uart_str = FDEV_SETUP_STREAM(uart_putchar, uart_getchar, _FDEV_SETUP_RW);

char data[16];

#define BRANCH_NODE 0b00100000
#define LEFT_TURN_NODE 0b01000000
#define RIGHT_TURN_NODE 0b01100000
#define END_NODE 0b10000000
#define STRAIGHT_NODE 0b10100000
#define OBSTRUCTION 0b11000000

int main(void) {
	unsigned char i;
	unsigned char j;
	char transmission_result;
	unsigned char pipe_received;
	uart_init();
	stdout = stdin = stderr = &uart_str;
	PAYLOAD_LENGTH = 2; //set the payload length of each packet
	init_RF(0); //init transceiver, disable enhanced shockburst
	set_mode(TX_MODE);
	set_RX_addr(PIPE_ADDRESS_0, 0);
	set_TX_addr(PIPE_ADDRESS_0);

	DDRD |= (1<<2);
	PORTD &= ~(1<<2);

	char* packet = (char *) malloc(PAYLOAD_LENGTH);
	char cmd_number = 0;
	while (1) {
		fscanf(stdin, "%s", data);

		if (strcmp(data,"go") == 0) {
			packet[0] = 0x01;
		}
		else if (strcmp(data,"tl") == 0) {
			packet[0] = 0x02;
		}
		else if (strcmp(data,"tr") == 0) {
			packet[0] = 0x03;
		}
		else if (strcmp(data,"ta") == 0) {
			packet[0] = 0x04;
		}
		else if (strcmp(data,"st") == 0) {
			packet[0] = 0x05;
		}

		packet[1] = cmd_number;

		transmission_result = transmit_packet(packet);
		if (transmission_result == 2) fprintf(stdout,"transmission failed\n");

		set_mode(RX_MODE);

		_delay_ms(5);

		do {
			while (RX_FIFO_status() != 1) {
		    set_mode(TX_MODE);
				transmission_result = transmit_packet(packet);
				if (transmission_result == 2) fprintf(stdout,"transmission failed\n");
				_delay_ms(1);
				set_mode(RX_MODE);
				_delay_ms(1);
			}
			pipe_received = read_RX_payload(packet);
		} while (packet[1] != cmd_number);

		if (packet[0] == BRANCH_NODE) {
            fprintf(stdout,"bn\n");
		}
		else if (packet[0] == LEFT_TURN_NODE) {
		    fprintf(stdout,"ln\n");
		}
		else if (packet[0] == RIGHT_TURN_NODE) {
			fprintf(stdout,"rn\n");
		}
		else if (packet[0] == END_NODE) {
			fprintf(stdout,"en\n");
		}
		else if (packet[0] == STRAIGHT_NODE) {
			fprintf(stdout,"sn\n");
		}
		else if (packet[0] == OBSTRUCTION) {
			fprintf(stdout,"ob\n");
		}
		else if (packet[0] == 0x02 || packet[0] == 0x03 || packet[0] == 0x04 || packet[0] == 0x05 || packet[0] == 0x06) {
		   fprintf(stdout,"ok\n");

		}

		set_mode(TX_MODE);
		cmd_number++;
	}

}
